<?php
namespace Mewz\WCAS\Actions\Common;

use Mewz\Framework\Base\Action;
use Mewz\WCAS\Models\AttributeStock;
use Mewz\WCAS\Util;

class StockChange extends Action
{
	public function __hooks()
	{
		// trigger when attribute stock is saved/changed/deleted
		add_action('mewz_attribute_stock_before_save', [$this, 'trigger'], 100);
		add_action('mewz_attribute_stock_saved', [$this, 'trigger'], 100);

		add_action('pre_post_update', [$this, 'pre_post_update'], 10, 2);
		add_action('save_post_mewz_attribute_stock', [$this, 'trigger'], 100);

		add_action('before_delete_post', [$this, 'before_delete_post'], 10, 2);

		// trigger when attributes/terms are deleted
		add_action('pre_delete_term', [$this, 'pre_delete_term'], 10, 2);
		add_action('woocommerce_before_attribute_delete', [$this, 'before_attribute_delete']);
	}

	public function pre_post_update($post_id, $data)
	{
		if ($data['post_type'] === 'mewz_attribute_stock') {
			$this->trigger($post_id);
		}
	}

	public function before_delete_post($post_id, $post = null)
	{
		$post_type = $post ? $post->post_type : get_post_type($post_id);

		if ($post_type=== 'mewz_attribute_stock') {
			$this->trigger($post_id);
		}
	}

	public function pre_delete_term($term_id, $taxonomy)
	{
		if (strpos($taxonomy, 'pa_') !== 0) return;

		$attribute_id = Util\Attributes::get_attribute_id($taxonomy);
		if (!$attribute_id) return;

		$stock_ids = Util\Matches::query()
			->where('r.attribute_id', $attribute_id)
			->where('r.term_id', $term_id)
			->col('s.stock_id');

		foreach ($stock_ids as $stock_id) {
			$this->trigger($stock_id);
		}
	}

	public function before_attribute_delete($attribute_id)
	{
		$stock_ids = Util\Matches::query()
			->where('r.attribute_id', $attribute_id)
			->col('s.stock_id');

		foreach ($stock_ids as $stock_id) {
			$this->trigger($stock_id);
		}
	}

	public function trigger($stock_id)
	{
		if ($stock_id instanceof AttributeStock) {
			$stock_id = $stock_id->id();
		}

		// triggers whenever a stock item changes or is about to change
		// (e.g. added, updated, deleted, attributes changed)
	    do_action('mewz_wcas_stock_item_change', $stock_id);
	}
}
