<?php
namespace Mewz\WCAS\Util;

class Remote
{
	/**
	 * @param string $action
	 * @param array $data
	 * @param array $args
	 *
	 * @return array|\WP_Error
	 */
	public static function background_action($action, $data = [], $args = [])
	{
		$url = admin_url('admin-ajax.php?action=' . $action);

		if (empty($data['nonce'])) {
			$data['nonce'] = self::create_site_nonce($action);
		}

		$args += [
			'body' => $data,
			//'cookies' => $_COOKIE,
			'timeout' => 0.01,
			'blocking' => false,
			'sslverify' => apply_filters('https_local_ssl_verify', false),
		];

		return wp_remote_post($url, $args);
	}

	public static function create_site_nonce($action = '')
	{
		return substr(wp_hash(wp_nonce_tick() . '|' . $action, 'nonce'), -12, 10);
	}

	public static function verify_site_nonce($nonce, $action = '')
	{
		$i = wp_nonce_tick();

		// nonce generated 0-12 hours ago
		$expected = substr(wp_hash($i . '|' . $action, 'nonce'), -12, 10);
		if (hash_equals($expected, $nonce)) return 1;

		// nonce generated 12-24 hours ago
		$expected = substr(wp_hash(($i - 1) . '|' . $action, 'nonce'), -12, 10);
		if (hash_equals($expected, $nonce)) return 2;

		do_action('wp_verify_nonce_failed', $nonce, $action, 0, '');

		return false;
	}

	public static function check_site_nonce($action = '', $key = 'nonce')
	{
		if (!empty($_REQUEST[$key])) {
			$result = self::verify_site_nonce($_REQUEST[$key], $action);
		} else {
			$result = false;
		}

		if (!$result) {
			$context = Mewz_WCAS()->context;

			if ($context->ajax || $context->rest || $context->cron || $context->cli) {
				wp_die(-1, 403);
			} else {
				wp_nonce_ays($action);
			}
		}
	}

	public static function prepare_background_execution()
	{
		// attempt to keep running if user aborts the request
		ignore_user_abort(true);

		// attempt to close the client connection and continue in the background
		if (function_exists('fastcgi_finish_request') && PHP_VERSION_ID >= 70016) {
			if (!headers_sent()) {
				header('Expires: Wed, 11 Jan 1984 05:00:00 GMT');
				header('Cache-Control: no-cache, must-revalidate, max-age=0');
			}

			fastcgi_finish_request();
		}

		// don't lock up other requests while processing
		if (function_exists('session_write_close')) {
			session_write_close();
		}

		// attempt to allow the request to run longer than usual
		set_time_limit(900);

		// attempt to allow the request to use more memory
		ini_set('memory_limit', -1);
	}
}
